import React from 'react'
import ReactDOM from 'react-dom'
import { bindActionCreators } from 'redux'
import { Provider, connect } from 'react-redux'
import { Router, Route, IndexRoute, IndexRedirect, Link, hashHistory } from 'react-router'
import { routerActions } from 'react-router-redux'
import { UserAuthWrapper } from 'redux-auth-wrapper'

// Root reducer
import rootReducer from './reducers/index'

// Action creators
import * as actionCreators from './actions/actionCreators'

// Store
import store, { history } from './store'

// Components
import NavBar from './components/navbar.jsx'
import MovieList from './components/movie-list.jsx'
import UserLoginForm from './components/user-login.jsx'

class Main extends React.Component {
  componentWillMount() {
    this.props.isUserLoggedIn()
  }
  render() {
    return (
      <div>
        <div className="navbar navbar-inverse navbar-fixed-top">
          <NavBar {...this.props}/>
        </div>
        <div className="container-fluid">
          <div className="container">
            {React.cloneElement(this.props.children, this.props)}
          </div>
        </div>
      </div>
    );
  }
}

function mapStateToProps(state) {
  return {
    movieStore: state.movieStore,
    userStore: state.userStore,
  }
}

function mapDispatchToProps(dispatch) {
  return bindActionCreators(actionCreators, dispatch);
}

const App = connect(mapStateToProps, mapDispatchToProps)(Main);

// Redirects to /login by default
const UserIsAuthenticated = UserAuthWrapper({
  authSelector: state => state.userStore,
  redirectAction: routerActions.replace,
  wrapperDisplayName: 'UserIsAuthenticated',
  predicate: user => user.isLogged,
  failureRedirectPath: '/users/login',
})
const Authenticated = UserIsAuthenticated((props) => props.children);

ReactDOM.render((
  <Provider store={store}>
    <Router history={history}>
      <Route path="/" component={App}>
        <IndexRedirect to="/users/login" />
        <Route path="/users/login" component={UserLoginForm} />
        <Route component={Authenticated}>
          <Route path="/movies/popular" component={MovieList} />
        </Route>
      </Route>
    </Router>
  </Provider>
),document.getElementById('app'));
